/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvControlIf.h"
#include "HvSignalEnvelope.h"
#include "HvSignalVar.h"
#include "HvControlBinop.h"
#include "HvControlCast.h"
#include "HvControlVar.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_9Mu41fg7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_K9gIe1Hy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_o30dX7hM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_IBbplrik_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ZqtxBsuw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_HhRaX2z4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iLSAbk5u_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_iFWoePX4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_t2jRYskN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_j04Gmxdv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_peb1nbyB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_1R94XVeN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_YgXvbKNs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_V3kCCmbV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_RqEAtSpq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_t6cqk4OX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_PpAyoa2L_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_dTADapkB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_OZkmIoQ4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YDBoYIyV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_LYJYhZSD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YzwfbXkD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_JYuRhPFD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_HnP8kEky_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ooZK0XqF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_IyxVGBre_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_oZBOvQz2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_adybgi2Y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_UFQSqbux_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_DvBFqJOA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_zfhU0w9O_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_a8VVNHAR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_2WpQArV3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_QMaoUpF0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_xha4zdz3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_1411nwkV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Kx6lxfWv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_U5xt5IL4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Y4oUJddQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_q4eX11Ut_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ryuNgVcD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_1tEjpWaf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_XhdoSUGf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_UjgMe0YN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_e4xgo6t3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_4EU3Ryug_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_onHYjOcZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_nOfuoMkX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_L3nTGFgM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_71PAvI0f_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_556uwscc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_HtV6C8tj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_sQ9v2STx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_3CVOEecJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_xhqEFppB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Jh4xwGfn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_xKfXxG3h_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_XuZikWo2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_HJa1rkZQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_7npYa3vN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_lSzj5nyk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_uYZ4pVja_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_gipWi3gz;
  SignalEnvelope sEnv_dTADapkB;
  SignalEnvelope sEnv_OZkmIoQ4;
  SignalEnvelope sEnv_xha4zdz3;
  SignalEnvelope sEnv_1411nwkV;
  SignalLine sLine_jlyGTlWE;
  SignalLine sLine_v4SydzzZ;
  ControlBinop cBinop_9Mu41fg7;
  ControlBinop cBinop_K9gIe1Hy;
  ControlBinop cBinop_o30dX7hM;
  ControlBinop cBinop_IBbplrik;
  ControlBinop cBinop_iLSAbk5u;
  ControlBinop cBinop_t2jRYskN;
  ControlBinop cBinop_j04Gmxdv;
  ControlBinop cBinop_peb1nbyB;
  ControlVar cVar_1R94XVeN;
  ControlVar cVar_V3kCCmbV;
  SignalVarf sVarf_qwyBai69;
  ControlVar cVar_RqEAtSpq;
  ControlIf cIf_t6cqk4OX;
  ControlIf cIf_PpAyoa2L;
  ControlBinop cBinop_YDBoYIyV;
  ControlBinop cBinop_YzwfbXkD;
  ControlBinop cBinop_JYuRhPFD;
  ControlBinop cBinop_IyxVGBre;
  ControlBinop cBinop_adybgi2Y;
  SignalVarf sVarf_qRdIj6a6;
  ControlVar cVar_a8VVNHAR;
  ControlIf cIf_2WpQArV3;
  ControlIf cIf_QMaoUpF0;
  ControlBinop cBinop_Kx6lxfWv;
  ControlBinop cBinop_Y4oUJddQ;
  ControlBinop cBinop_q4eX11Ut;
  ControlBinop cBinop_XhdoSUGf;
  ControlBinop cBinop_e4xgo6t3;
  ControlVar cVar_L3nTGFgM;
  ControlBinop cBinop_71PAvI0f;
  SignalVarf sVarf_ao3T0tfU;
  SignalVarf sVarf_vTYGQ3Z6;
  SignalVarf sVarf_bwXUOBsg;
  SignalVarf sVarf_DrJyuvVs;
  ControlBinop cBinop_3CVOEecJ;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
